/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.collision;

import java.util.List;

import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.ChartArtifact;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.model.Calculation;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.DataFacet;
import org.dive4elements.river.artifacts.model.EmptyFacet;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.ReportFacet;
import org.dive4elements.river.artifacts.sinfo.SINFOArtifact;
import org.dive4elements.river.artifacts.sinfo.common.CollisionCalcProcessor;
import org.dive4elements.river.artifacts.states.DefaultState;

/**
 * State in which a collision count has been calculated.
 */
public class CollisionState extends DefaultState {

    /// ** The log that is used in this state. */
    // private static Logger log = Logger.getLogger(CollisionState.class);

    private static final long serialVersionUID = 1L;

    /**
     * From this state can only be continued trivially.
     */
    @Override
    protected String getUIProvider() {
        return "continue";
    }

    @Override
    public Object computeFeed(final D4EArtifact artifact, final String hash, final CallContext context, final List<Facet> facets, final Object old) {
        if (artifact instanceof ChartArtifact) {
            facets.add(new EmptyFacet());
            return null;
        }
        return compute((SINFOArtifact) artifact, context, hash, facets, old);
    }

    @Override
    public Object computeAdvance(final D4EArtifact artifact, final String hash, final CallContext context, final List<Facet> facets, final Object old) {
        if (artifact instanceof ChartArtifact) {
            facets.add(new EmptyFacet());
            return null;
        }
        return compute((SINFOArtifact) artifact, context, hash, facets, old);
    }

    /**
     * Compute result or returned object from cache, create facets.
     *
     * @param old
     *            Object that was cached.
     */
    private Object compute(final SINFOArtifact sinfo, final CallContext context, final String hash, final List<Facet> facets, final Object old) {

        final CalculationResult res = doCompute(sinfo, context, old);

        if (facets == null)
            return res;
        //
        final CollisionCalculationResults results = (CollisionCalculationResults) res.getData();

        /* add themes for chart, for each result */
        final List<AbstractCalculationExportableResult<CollisionCalculationResults>> resultList = results.getResults();
        for (int index = 0; index < resultList.size(); index++) {
            if (resultList.get(index) instanceof CollisionCalcOverviewResult) {
                final CollisionCalcOverviewResult result = (CollisionCalcOverviewResult) resultList.get(index);
                facets.add(CollisionCalcProcessor.createFacet(context, hash, this.id, result, index));
            }
        }

        if (!resultList.isEmpty()) {
            final Facet csv = new DataFacet(FacetTypes.CSV, "CSV data", ComputeType.ADVANCE, hash, this.id);
            final Facet pdf = new DataFacet(FacetTypes.PDF, "PDF data", ComputeType.ADVANCE, hash, this.id);

            facets.add(csv);
            facets.add(pdf);
        }

        final Calculation report = res.getReport();

        if (report.hasProblems()) {
            facets.add(new ReportFacet(ComputeType.ADVANCE, hash, this.id));
        }

        return res;
    }

    private CalculationResult doCompute(final SINFOArtifact sinfo, final CallContext context, final Object old) {
        if (old instanceof CalculationResult)
            return (CalculationResult) old;

        return new CollisionCalculation(context).calculate(sinfo);
    }
}